﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.BinaryVectorEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.FitnessLandscape {

  [Item("BinaryVectorFitnessDistanceCorrelationAnalyzer", "An operator that analyzes the correlation between fitness and distance to the best know solution for binary vector encoding")]
  [StorableClass]
  public class BinaryVectorFitnessDistanceCorrelationAnalyzer : FitnessDistanceCorrelationAnalyzer, IBinaryVectorOperator {

    #region Parameters
    public ScopeTreeLookupParameter<BinaryVector> BinaryVectorParameter {
      get { return (ScopeTreeLookupParameter<BinaryVector>)Parameters["BinaryVector"]; }
    }
    public LookupParameter<BinaryVector> BestKnownSolution {
      get { return (LookupParameter<BinaryVector>)Parameters["BestKnownSolution"]; }
    }
    #endregion

    [StorableConstructor]
    protected BinaryVectorFitnessDistanceCorrelationAnalyzer(bool deserializing) : base(deserializing) { }
    protected BinaryVectorFitnessDistanceCorrelationAnalyzer(BinaryVectorFitnessDistanceCorrelationAnalyzer original, Cloner cloner) : base(original, cloner) { }

    public BinaryVectorFitnessDistanceCorrelationAnalyzer() {
      Parameters.Add(new ScopeTreeLookupParameter<BinaryVector>("BinaryVector", "The real encoded solution"));
      Parameters.Add(new LookupParameter<BinaryVector>("BestKnownSolution", "The best known solution"));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new BinaryVectorFitnessDistanceCorrelationAnalyzer(this, cloner);
    }

    public static double Distance(BinaryVector a, BinaryVector b) {
      if (a.Length != b.Length)
        throw new InvalidOperationException("Cannot compare vectors of different lengths");
      double nEqualBits = 0;
      for (int i = 0; i < a.Length; i++) {
        if (a[i] == b[i])
          nEqualBits++;
      }
      return Math.Max(a.Length, b.Length) - nEqualBits;
    }


    protected override IEnumerable<double> GetDistancesToBestKnownSolution() {
      if (BinaryVectorParameter.ActualValue == null)
        return new double[0];
      BinaryVector bestKnownValue = BestKnownSolution.ActualValue;
      if (bestKnownValue == null)
        return BinaryVectorParameter.ActualValue.Select(v => 0d);
      return BinaryVectorParameter.ActualValue.Select(v => Distance(v, bestKnownValue));
    }
  }
}